#!/bin/bash
#

# buildXmlrpc-c.sh <product_dir> <e10|e14|e15> <debug|prof> [tar] [source]

# IMPORTANT: If curl-devel (SLF5) or libcurl-devel (SLF6) is installed, 
#            the curl xml client will also be built

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e-qualifier> <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=xmlrpc_c
origpkgver=v1_51_06
pkgver=${origpkgver}
ssibuildshims_version=v2_07_00
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=xmlrpc-c-${pkgdotver}

get_this_dir

get_ssibuildshims

source define_basics --

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "opt" ]
then
    cflg="-O3 -g -DNDEBUG"
elif [ "${extraqual}" = "debug" ]
then
    cflg="-g -O0"
elif [ "${extraqual}" = "prof" ]
then
    cflg="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
fi

if [[ "${basequal}" == e1[045] ]]; then
  cxxflg="${cflg} -std=c++14 -Wno-deprecated-declarations"
  compiler=gcc
  export CC=gcc
  export CXX=g++
  extra_command="stdver=c++14 ${extra_command}"
elif [[ "${basequal}" == e1[79] ]]; then
  cxxflg="${cflg} -std=c++17 -Wno-deprecated-declarations -D_LIBCPP_ENABLE_CXX17_REMOVED_FEATURES=1"
  compiler=gcc
  export CC=gcc
  export CXX=g++
  extra_command="stdver=c++17 ${extra_command}"
elif [[ "${basequal}" == e20 ]]; then
  cxxflg="${cflg} -std=c++17 -Wno-deprecated-declarations -D_LIBCPP_ENABLE_CXX17_REMOVED_FEATURES=1"
  compiler=gcc
  export CC=gcc
  export CXX=g++
  extra_command="stdver=c++17 ${extra_command}"
elif [[ "${basequal}" == c[27] ]]; then
  compiler=clang
  export CC=clang
  export CXX=clang++
  cxxflg="${cflg} -std=c++17 -Wno-deprecated-declarations -D_LIBCPP_ENABLE_CXX17_REMOVED_FEATURES=1"
  extra_command="stdver=c++17 ${extra_command}"
#elif [[ "${basequal}" == "i1" ]]; then
#  compiler=icc
#  extra_command="stdver=c++11 ${extra_command}"
else
  ssi_die "ERROR: unrecognized qualifier $basequal."
fi

if [ -d ${pkgdir} ]; then rm -rf ${pkgdir}; fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# setup root
setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "ERROR: fake setup failed"

# XMLRPC_C as used by ARTDAQ ALWAYS requires curl client...
#have_curl=`rpm -qa | grep curl-devel | wc -l` # Platform-dependent nastiness aside...
have_curl=1
if [ ${have_curl} -gt 0 ]
then
   build_curl=--enable-curl-client 
   echo "building xmlrpc_c with curl client"
else
   build_curl=--disable-curl-client 
   echo "building xmlrpc_c WITHOUT curl client"
fi

set -x

cd ${blddir}
tar xf ${tardir}/${pkgtarname}.tgz

# Apply patch:
for patchfile in ${patchdir}/*;do
    #patchfile=${patchdir}/xmlrpc_string.patch
    patch ${DEBUG_PATCHES:+--verbose} -p0 < "${patchfile}" || ssi_die "ERROR: application of patch ${patchfile} failed."
done

cd ${blddir}/${pkgtarname}

./configure --prefix=${pkgdir} \
            ${build_curl} --enable-cplusplus \
	    --disable-libwww-client --disable-wininet-client || die "ERROR: configure failed."
make compiler=${compiler} ${extra_command} CFLAGS_LOCAL="${cflg}" CXXFLAGS="${cxxflg}" all || die "ERROR: make failed."
## disable make check because the abyss.c test file explicitly uses port 8080 which is not always available
##make CFLAGS_LOCAL="${cflg}" CXXFLAGS="${cxxflg}" check || die "ERROR: make check failed."
make compiler=${compiler} ${extra_command} CFLAGS_LOCAL="${cflg}" CXXFLAGS="${cxxflg}" install || die "ERROR: make install failed."
# install the tools
cd ${blddir}/${pkgtarname}/tools; make CFLAGS_LOCAL="${cflg}" CXXFLAGS="${cxxflg}" install || die "ERROR: make install of tools failed."

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "xmlrpc_c libraries are in ${XMLRPC_C_LIBRARY}"

# fixing embedded path
mv ${XMLRPC_C_FQ_DIR}/bin/xmlrpc-c-config ${XMLRPC_C_FQ_DIR}/bin/xmlrpc-c-config.bak
cat ${XMLRPC_C_FQ_DIR}/bin/xmlrpc-c-config.bak \
    | sed 's%^PREFIX=.*$%PREFIX=${XMLRPC_C_FQ_DIR}%' \
    | sed 's%^HEADERINST_DIR=.*$%HEADERINST_DIR=${XMLRPC_C_INCLUDE_DIR}%' \
    | sed 's%^LIBINST_DIR=.*$%LIBINST_DIR=${XMLRPC_C_LIBRARY}%' \
    | sed 's%^BLDDIR=.*$%BLDDIR=${XMLRPC_C_FQ_DIR}/build/stable%' \
    | sed 's%^ABS_SRCDIR=.*$%ABS_SRCDIR=${XMLRPC_C_FQ_DIR}/build/stable%' \
    > ${XMLRPC_C_FQ_DIR}/bin/xmlrpc-c-config
chmod +x ${XMLRPC_C_FQ_DIR}/bin/xmlrpc-c-config

# this must be last 
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
